@@ -1,6 +1,6 @@
 ;;; ob-lisp.el --- org-babel functions for common lisp evaluation
 
-;; Copyright (C) 2009-2013 Free Software Foundation, Inc.
+;; Copyright (C) 2009-2014 Free Software Foundation, Inc.
 
 ;; Authors: Joel Boehland
 ;;	 Eric Schulte
@@ -44,7 +44,7 @@
 (defvar org-babel-header-args:lisp '((package . :any)))
 
 (defcustom org-babel-lisp-dir-fmt
-  "(let ((*default-pathname-defaults* #P%S)) %%s)"
+  "(let ((*default-pathname-defaults* #P%S\n)) %%s\n)"
   "Format string used to wrap code bodies to set the current directory.
 For example a value of \"(progn ;; %s\\n   %%s)\" would ignore the
 current directory string."
@@ -75,23 +75,26 @@ current directory string."
   "Execute a block of Common Lisp code with Babel."
   (require 'slime)
   (org-babel-reassemble-table
-   ((lambda (result)
-      (org-babel-result-cond (cdr (assoc :result-params params))
-	(car result)
-	(condition-case nil
-	    (read (org-babel-lisp-vector-to-list (cadr result)))
-	  (error (cadr result)))))
-    (with-temp-buffer
-      (insert (org-babel-expand-body:lisp body params))
-      (slime-eval `(swank:eval-and-grab-output
-		    ,(let ((dir (if (assoc :dir params)
-				    (cdr (assoc :dir params))
-				  default-directory)))
-		       (format
-			(if dir (format org-babel-lisp-dir-fmt dir) "(progn %s)")
-			(buffer-substring-no-properties
-			 (point-min) (point-max)))))
-		  (cdr (assoc :package params)))))
+   (let ((result
+	  (funcall (if (member "output" (cdr (assoc :result-params params)))
+		       #'car #'cadr)
+		   (with-temp-buffer
+		     (insert (org-babel-expand-body:lisp body params))
+		     (slime-eval `(swank:eval-and-grab-output
+				   ,(let ((dir (if (assoc :dir params)
+						   (cdr (assoc :dir params))
+						 default-directory)))
+				      (format
+				       (if dir (format org-babel-lisp-dir-fmt dir)
+					 "(progn %s\n)")
+				       (buffer-substring-no-properties
+					(point-min) (point-max)))))
+				 (cdr (assoc :package params)))))))
+     (org-babel-result-cond (cdr (assoc :result-params params))
+       result
+       (condition-case nil
+           (read (org-babel-lisp-vector-to-list result))
+         (error result))))
    (org-babel-pick-name (cdr (assoc :colname-names params))
 			(cdr (assoc :colnames params)))
    (org-babel-pick-name (cdr (assoc :rowname-names params))
