@@ -1,12 +1,10 @@
 ;;; org-info.el --- Support for links to Info nodes from within Org-Mode
 
-;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
-;;   Free Software Foundation, Inc.
+;; Copyright (C) 2004-2014 Free Software Foundation, Inc.
 
 ;; Author: Carsten Dominik <carsten at orgmode dot org>
 ;; Keywords: outlines, hypermedia, calendar, wp
 ;; Homepage: http://orgmode.org
-;; Version: 7.7
 ;;
 ;; This file is part of GNU Emacs.
 ;;
@@ -42,7 +40,7 @@
 (defvar Info-current-node)
 
 ;; Install the link type
-(org-add-link-type "info" 'org-info-open)
+(org-add-link-type "info" 'org-info-open 'org-info-export)
 (add-hook 'org-store-link-functions 'org-info-store-link)
 
 ;; Implementation
@@ -50,9 +48,9 @@
   "Store a link to an Info file and node."
   (when (eq major-mode 'Info-mode)
     (let (link desc)
-      (setq link (org-make-link "info:"
-				(file-name-nondirectory Info-current-file)
-				"#" Info-current-node))
+      (setq link (concat "info:"
+			 (file-name-nondirectory Info-current-file)
+			 "#" Info-current-node))
       (setq desc (concat (file-name-nondirectory Info-current-file)
 			 "#" Info-current-node))
       (org-store-link-props :type "info" :file Info-current-file
@@ -69,15 +67,33 @@
   "Follow an Info file and node link specified by NAME."
   (if (or (string-match "\\(.*\\)[#:]:?\\(.*\\)" name)
           (string-match "\\(.*\\)" name))
-      (progn
+      (let ((filename (match-string 1 name))
+	    (nodename-or-index (or (match-string 2 name) "Top")))
 	(require 'info)
-        (if (match-string 2 name) ; If there isn't a node, choose "Top"
-            (Info-find-node (match-string 1 name) (match-string 2 name))
-          (Info-find-node (match-string 1 name) "Top")))
-    (message "Could not open: %s" name)))
+	;; If nodename-or-index is invalid node name, then look it up
+	;; in the index.
+	(condition-case nil
+	    (Info-find-node filename nodename-or-index)
+	  (user-error (Info-find-node filename "Top")
+		      (condition-case nil
+			  (Info-index nodename-or-index)
+			(user-error "Could not find '%s' node or index entry"
+				    nodename-or-index)))))
+    (user-error "Could not open: %s" name)))
+
+(defun org-info-export (path desc format)
+  "Export an info link.
+See `org-add-link-type' for details about PATH, DESC and FORMAT."
+  (when (eq format 'html)
+    (or (string-match "\\(.*\\)[#:]:?\\(.*\\)" path)
+	(string-match "\\(.*\\)" path))
+    (let ((filename (match-string 1 path))
+	  (node (or (match-string 2 path) "Top")))
+      (format "<a href=\"%s.html#%s\">%s</a>"
+	      filename
+	      (replace-regexp-in-string " " "-" node)
+	      (or desc path)))))
 
 (provide 'org-info)
 
-;; arch-tag: 1e289f54-7176-487f-b575-dd4854bab15e
-
 ;;; org-info.el ends here
